@@ -106,7 +106,7 @@ urlopener = create_url_opener()
 
 def quote_url_path(url):
     p = urlsplit(url)
-    return urlunsplit((p[0], p[1], urllib.quote(p[2]), p[3], p[4]))
+    return urlunsplit((p[0], p[1], urllib.quote(p[2], "/%"), p[3], p[4]))
 
 
 def get_or_create_tag(html, list, tag = None):
@@ -122,6 +122,15 @@ def get_or_create_tag(html, list, tag = None):
         return sub
 
 
+def tag_contains(tag, another):
+    if tag:
+        while another:
+            if tag is another:
+                return True
+            another = another.parent
+    return False
+
+
 def get_page_encoding(page):
     """ Not all html documents has http-equiv meta, so this retrieves real http headers.
     """
@@ -181,6 +190,7 @@ def get_page_html(url, **kwargs):
     #(page, html, base) = try_skip_banner(page, html, base, **kwargs)
     return (html, base, page.geturl())
 
+
 def get_feed_title(xml):
     chan = xml.find(["channel", "feed"])
     if chan:
@@ -256,7 +266,7 @@ class Item:
 
     def __init__(self, url = None, tag = None, base = None, path = "", cache_path = "", callback = None, \
         page_template = None, print_version_keywords = None, try_use_print_version = True, \
-        allow_scripts = False, as_is = False, content_kwargs = None, reformatter_url = None):
+        allow_scripts = False, as_is = False, content_kwargs = None, reformatter_url = None, next_page_keyword = None):
         """ It is clear that this is constructor :)
 
         Item is represented either by its content, when tag is specified, or by reference
@@ -274,6 +284,7 @@ class Item:
         content_kwargs - it is passed to BeautifulSoup's find method. It is usually
         tag name and its attributes.
         reformatter_url - format string (%s is replaced with url from item).
+        next_page_keyword - the name of the link that leads to the next page of document if any
         Used to process html document through online services such as Skweezer.
         """
         self.__url = url
@@ -298,6 +309,8 @@ class Item:
         self.__as_is = as_is
         self.__content_kwargs = content_kwargs
         self.__reformatter_url = reformatter_url
+        self.__next_page_keyword = next_page_keyword
+        self.__next_page_a = None
 
 
     def __report(self, action, item):
@@ -335,6 +348,24 @@ class Item:
         return cache_path
 
 
+    def __cache_next_page(self, html, base):
+        try:
+            text = html.find(text=lambda t: t.strip().lower() == self.__next_page_keyword.strip().lower())
+            if text:
+                a = text.findParent(lambda t: t.name == "a" and t.has_key("href"))
+                if a:
+                    url = urljoin(base, a["href"])
+                    item = Item(url = url, path = self.__path, cache_path = self.__cache_path,
+                        callback = self.__callback, page_template = self.__page_template,
+                        try_use_print_version = False, allow_scripts = self.__allow_scripts,
+                        content_kwargs = self.__content_kwargs, next_page_keyword = self.__next_page_keyword)
+                    item.process()
+                    a["href"] = item.save()
+                    return a
+        except Exception, ex:
+            self.__report("error", ex)
+
+
     def __find_print_version(self, html, base, referer):
         try:
             text = html.find(text=lambda t: t.strip().lower() in self.__print_version_keywords)
@@ -352,7 +383,7 @@ class Item:
 
     def get_url_for_reformatter(self, url):
         if self.__reformatter_url:
-            return self.__reformatter_url % urllib.quote(url)
+            return self.__reformatter_url % urllib.quote(url, "%")
         return url
 
 
@@ -366,6 +397,8 @@ class Item:
         if not self.__as_is:
             if self.__try_use_print_version:
                 (html, self.__base, self.__tag) = self.__find_print_version(html, self.__base, real_url)
+            if not self.__tag and self.__next_page_keyword:
+                self.__next_page_a = self.__cache_next_page(html, self.__base)
             if not self.__reformatter_url:
                 if self.__content_kwargs:
                     self.__tag = html.body.find(**self.__content_kwargs)
@@ -384,7 +417,11 @@ class Item:
 
 
     def __fix_urls(self, tags, attr, base):
+        if self.__next_page_a:
+            href = self.__next_page_a["href"]
         [self.__fix_url(tag, attr, base) for tag in tags]
+        if self.__next_page_a:
+            self.__next_page_a["href"] = href
 
 
     def __cache_url(self, tag, attr, base):
@@ -395,7 +432,7 @@ class Item:
                 if self.__cached.has_key(url):
                     path_for_url = self.__cached[url]
                 else:
-                    filename = uuid.uuid1().hex + urlparse(url)[2].split("/")[-1]
+                    filename = uuid.uuid1().hex + urllib.unquote(urlparse(url)[2].split("/")[-1])
                     path_for_url = os.path.join(self.__cache_path, filename)
                     tag.parent.append(Comment(NavigableString(self.cached_comment_prefix + path_for_url)))
                     self.__report("downloading", url)
@@ -423,10 +460,17 @@ class Item:
         [self.__fix_urls(self.__tag.findAll(x[0]), x[1], self.__base) for x in self.__attrs_for_fix]
         [self.__cache_urls(self.__tag.findAll(x[0]), x[1], self.__base) for x in self.__attrs_for_cache]
         if self.__tag.name in ["table", "dir", "menu", "dl", "ul", "ol"]:
-            return [self.__tag]
+            content = [self.__tag]
         elif self.__tag.name in ["tr", "caption", "colgroup", "thead", "tfoot", "tbody"]:
-            return [self.__tag.findParent("table")]
-        return [node for node in self.__tag.contents]
+            content = [self.__tag.findParent("table")]
+        else:
+            content = [node for node in self.__tag.contents]
+        if self.__next_page_a:
+            content.append(Comment(NavigableString(self.cached_comment_prefix + self.__next_page_a["href"])))
+            if not tag_contains(self.__tag, self.__next_page_a):
+                self.__next_page_a.extract()
+                content.append(self.__next_page_a)
+        return content
 
 
     def save(self, title = None, filename = None):
@@ -494,7 +538,7 @@ class Feed:
         page_template = None, page_item_template = None, print_version_keywords = None, \
         try_use_print_version = True, delete_before_date = None, allow_scripts = False, \
         inline_items = False, cache_items = False, include = None, exclude = None, as_is = False,
-        content_kwargs = None, reformatter_url = None):
+        content_kwargs = None, reformatter_url = None, next_page_keyword = None):
         """ Constructor.
 
         url - feed url.
@@ -518,6 +562,7 @@ class Feed:
         content_kwargs - it is passed to BeautifulSoup's find method. It is usually
         tag name and its attributes.
         reformatter_url - format string (%s is replaced with url from item).
+        next_page_keyword - the name of the link that leads to the next page of document if any
         Used to process html document through online services such as Skweezer.
         """
         self.__url = url
@@ -547,6 +592,7 @@ class Feed:
         self.__as_is = as_is
         self.__content_kwargs = content_kwargs
         self.__reformatter_url = reformatter_url
+        self.__next_page_keyword = next_page_keyword
 
 
     def __report(self, action, item):
@@ -622,7 +668,8 @@ class Feed:
         return Item(url = url, tag = tag, base = base, path = path, cache_path = cache_path, \
             callback = self.__callback, try_use_print_version = self.__try_use_print_version, \
             print_version_keywords = self.__print_version_keywords, allow_scripts = self.__allow_scripts, \
-            as_is = self.__as_is, content_kwargs = self.__content_kwargs, reformatter_url = self.__reformatter_url)
+            as_is = self.__as_is, content_kwargs = self.__content_kwargs, \
+            reformatter_url = self.__reformatter_url, next_page_keyword = self.__next_page_keyword)
         
 
     def __add_item(self, div, item):
@@ -678,6 +725,7 @@ class Feed:
         Outdated items are deleted only when there are some new ones.
         """
         result = 0
+        div = None
         try:
             self.__report("downloading", self.__url)
             page = urlopener.open(quote_url_path(self.__url))
@@ -699,7 +747,9 @@ class Feed:
                 isinstance(text, Comment) and text.startswith(self.item_comment_prefix))
             div = Tag(html, "div")
             div["id"] = datetime.today().strftime("%Y%m%d%H%M%S")
-            div.append(NavigableString(unicode(datetime.today().strftime("%c").decode(locale.getpreferredencoding()))))
+            update_str = self.__title + ", " + \
+                unicode(datetime.today().strftime("%c").decode(locale.getpreferredencoding()))
+            div.append(NavigableString(update_str))
             div.append(Tag(html, "hr"))
             get_or_create_tag(html, ["body"]).insert(0, div)
             [self.__add_item(div, tag) for tag in xml.findAll(["item", "entry"])]
@@ -711,7 +761,8 @@ class Feed:
                 f.close()
         except Exception, ex:
             self.__report("error", ex)
-            self.__delete_div(div)
+            if div:
+                self.__delete_div(div)
         return result
 
 
