@@ -24,7 +24,7 @@ gtk.gdk.threads_init()
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.6", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.7", False)
 except ImportError:
     pass
 
@@ -136,7 +136,8 @@ class FeedConfig:
 
     def __init__(self, cache_path, url, name, mode, days_to_keep,
         include, exclude, ignore_print_version, icon_filename,
-        message_count, as_is, content_tag, print_version_link, reformatter_url, id = None):
+        message_count, as_is, content_tag, print_version_link, reformatter_url,
+        next_page_link, id = None):
         self.id = id
         if not self.id:
             self.id = uuid.uuid1().hex
@@ -154,6 +155,7 @@ class FeedConfig:
         self.content_tag = content_tag
         self.print_version_link = print_version_link
         self.reformatter_url = reformatter_url
+        self.next_page_link = next_page_link
         self.icon = None
         self.iter = None
         self.ord_num = 0
@@ -204,6 +206,10 @@ class FeedConfig:
         return result
 
 
+    def get_urls(self):
+        return filter(len, [str.strip() for str in self.url.split(" ")])
+
+
 class FeedListModel(gtk.GenericTreeModel):
 
 
@@ -302,6 +308,7 @@ class FeedListModel(gtk.GenericTreeModel):
                     parser.get_default(section, "content tag", ""),
                     parser.get_default(section, "print version link", ""),
                     parser.get_default(section, "reformatter url", ""),
+                    parser.get_default(section, "next page link", ""),
                     section)
                 feed_config.ord_num = parser.getint_default(section, "order number", 0)
                 self.list.append(feed_config)
@@ -331,6 +338,7 @@ class FeedListModel(gtk.GenericTreeModel):
                 parser.set(feed_config.id, "content tag", feed_config.content_tag)
                 parser.set(feed_config.id, "print version link", feed_config.print_version_link)
                 parser.set(feed_config.id, "reformatter url", feed_config.reformatter_url)
+                parser.set(feed_config.id, "next page link", feed_config.next_page_link)
                 parser.set(feed_config.id, "order number", str(i))
                 i += 1
             parser.write(f)
@@ -392,6 +400,27 @@ class FeedListModel(gtk.GenericTreeModel):
         self.save_config()
 
 
+class UrlsWindow(gtk.Dialog):
+
+
+    def __init__(self, parent):
+        gtk.Dialog.__init__(self, None, parent,
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
+             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        self.resize(*parent.get_size())
+        self.set_title(_("Feed urls"))
+        self.vbox.pack_start(gtk.Label(_("One url per line")), expand = False)
+        self.text_buffer = gtk.TextBuffer()
+        sw = gtk.ScrolledWindow()
+        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        sw.set_shadow_type(gtk.SHADOW_IN)
+        text_view = gtk.TextView(self.text_buffer)
+        sw.add(text_view)
+        self.vbox.pack_start(sw, expand = True)
+        self.show_all()
+
+
 class PreferencesWindow(gtk.Dialog):
 
 
@@ -491,9 +520,12 @@ class ContentWindow(gtk.Dialog):
         self.ignore_print_version_button = gtk.CheckButton()
         self.ignore_print_version_button.connect("toggled", self.something_changed)
         self.print_version_link_entry = gtk.Entry()
-        table = gtk.Table(rows = 5, columns = 2)
+        self.print_version_link_entry.connect("changed", self.something_changed)
+        self.next_page_link_entry = gtk.Entry()
+        self.next_page_link_entry.connect("changed", self.something_changed)
+        table = gtk.Table(rows = 6, columns = 2)
         texts = [_("Use as is"), _("Content tag"), _("Reformatting service url"),
-            _("Ignore printable version"), _("Printable version link") ]
+            _("Ignore printable version"), _("Printable version link"), _("Next page link") ]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
             label.set_alignment(1, 1)
@@ -503,6 +535,7 @@ class ContentWindow(gtk.Dialog):
         table.attach(self.reformatter_url_entry, 1, 2, 2, 3, xpadding = 3, ypadding = 3)
         table.attach(self.ignore_print_version_button, 1, 2, 3, 4, xpadding = 3, ypadding = 3)
         table.attach(self.print_version_link_entry, 1, 2, 4, 5, xpadding = 3, ypadding = 3)
+        table.attach(self.next_page_link_entry, 1, 2, 5, 6, xpadding = 3, ypadding = 3)
         self.vbox.add(table)
         self.show_all()
 
@@ -511,11 +544,15 @@ class ContentWindow(gtk.Dialog):
         self.ignore_print_version_button.set_sensitive(not self.as_is_button.get_active())
         self.print_version_link_entry.set_sensitive(
             not self.as_is_button.get_active() and
-            not self.ignore_print_version_button.get_active())
+            not self.ignore_print_version_button.get_active() and
+            len(self.next_page_link_entry.get_text()) == 0)
         self.content_tag_entry.set_sensitive(
             not self.as_is_button.get_active() and len(self.reformatter_url_entry.get_text()) == 0)
         self.reformatter_url_entry.set_sensitive(
             not self.as_is_button.get_active() and len(self.content_tag_entry.get_text()) == 0)
+        self.next_page_link_entry.set_sensitive(
+            not self.as_is_button.get_active() and
+            len(self.print_version_link_entry.get_text()) == 0)
 
 
 class EditWindow(gtk.Dialog):
@@ -536,13 +573,19 @@ class EditWindow(gtk.Dialog):
         self.content_tag = feed_config.content_tag
         self.print_version_link = feed_config.print_version_link
         self.reformatter_url = feed_config.reformatter_url
+        self.next_page_link = feed_config.next_page_link
+        hbox = gtk.HBox()
         self.url_entry = gtk.Entry()
         self.url_entry.set_width_chars(32)
         self.url_entry.set_text(feed_config.url)
+        self.edit_button = gtk.ToolButton(stock_id = gtk.STOCK_EDIT)
+        self.edit_button.connect("clicked", self.edit_button_clicked)
+        hbox.pack_start(self.url_entry, expand = True)
+        hbox.pack_start(self.edit_button, expand = False)
         self.name_entry = gtk.Entry()
         self.name_entry.set_text(feed_config.name)
-        hbox = gtk.HBox(spacing = 3)
-        self.content_button = gtk.Button(_("Advanced"))
+        hbox2 = gtk.HBox(spacing = 3)
+        self.content_button = gtk.ToolButton(gtk.STOCK_PROPERTIES)
         self.content_button.connect("clicked", self.content_clicked)
         self.mode_combo = gtk.combo_box_new_text()
         self.mode_combo.connect("changed", self.mode_changed)
@@ -550,8 +593,8 @@ class EditWindow(gtk.Dialog):
         self.mode_combo.append_text(_("Cache"))
         self.mode_combo.append_text(_("Inline"))
         self.mode_combo.set_active(feed_config.mode)
-        hbox.pack_start(self.mode_combo)
-        hbox.pack_start(self.content_button, expand = False)
+        hbox2.pack_start(self.mode_combo)
+        hbox2.pack_start(self.content_button, expand = False)
         self.extra_button = gtk.Button(_("More"))
         self.extra_button.connect("clicked", self.extra_clicked)
         table = gtk.Table(rows = 4, columns = 2)
@@ -560,12 +603,12 @@ class EditWindow(gtk.Dialog):
             label = gtk.Label(texts[i])
             label.set_alignment(1, 1)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
-        table.attach(self.url_entry, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
+        table.attach(hbox, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
         table.attach(self.name_entry, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
-        table.attach(hbox, 1, 2, 2, 3, xpadding = 3, ypadding = 3)
-        hbox = gtk.HBox()
-        hbox.pack_end(self.extra_button, expand = False)
-        table.attach(hbox, 1, 2, 3, 4, xpadding = 3, ypadding = 8)
+        table.attach(hbox2, 1, 2, 2, 3, xpadding = 3, ypadding = 3)
+        hbox3 = gtk.HBox()
+        hbox3.pack_end(self.extra_button, expand = False)
+        table.attach(hbox3, 1, 2, 3, 4, xpadding = 3, ypadding = 8)
         self.vbox.add(table)
         self.show_all()
 
@@ -586,6 +629,7 @@ class EditWindow(gtk.Dialog):
         self.feed_config.content_tag = self.content_tag
         self.feed_config.print_version_link = self.print_version_link
         self.feed_config.reformatter_url = self.reformatter_url
+        self.feed_config.next_page_link = self.next_page_link
 
 
     def extra_clicked(self, source):
@@ -607,15 +651,28 @@ class EditWindow(gtk.Dialog):
         cnt_window.content_tag_entry.set_text(self.content_tag)
         cnt_window.reformatter_url_entry.set_text(self.reformatter_url)
         cnt_window.print_version_link_entry.set_text(self.print_version_link)
+        cnt_window.next_page_link_entry.set_text(self.next_page_link)
         if cnt_window.run() == gtk.RESPONSE_ACCEPT:
             self.as_is = cnt_window.as_is_button.get_active()
             self.ignore_print_version = cnt_window.ignore_print_version_button.get_active()
             self.content_tag = cnt_window.content_tag_entry.get_text()
             self.reformatter_url = cnt_window.reformatter_url_entry.get_text()
             self.print_version_link = cnt_window.print_version_link_entry.get_text()
+            self.next_page_link = cnt_window.next_page_link_entry.get_text()
         cnt_window.destroy()
 
 
+    def edit_button_clicked(self, source):
+        urls_window = UrlsWindow(self)
+        tb = urls_window.text_buffer
+        tb.set_text(self.url_entry.get_text().replace(" ", "\n"))
+        if urls_window.run() == gtk.RESPONSE_ACCEPT:
+            text = tb.get_text(tb.get_start_iter(), tb.get_end_iter(), include_hidden_chars = False)
+            lines = filter(len, [line.strip().replace(" ", "%20") for line in text.split("\n")])
+            self.url_entry.set_text(reduce(lambda s1, s2: s1 + " " + s2, lines))
+        urls_window.destroy()
+
+
 class Program(hildon.Program):
 
 
@@ -838,18 +895,23 @@ class Program(hildon.Program):
             for feed_config in feed_configs:
                 i += 1
                 self.__progress_info = (i, len(feed_configs))
-                feed = feedcircuit.Feed(url = feed_config.url, path = self.__model.cache_path, cache_path = ".cache",
-                    title = feed_config.name, filename = feed_config.id + ".html",
-                    callback = self.callback, try_use_print_version = not feed_config.ignore_print_version,
-                    delete_before_date = date.today() - timedelta(feed_config.days_to_keep),
-                    inline_items = feed_config.mode == FeedConfig.mode_inline,
-                    cache_items = feed_config.mode == FeedConfig.mode_cache,
-                    include = feed_config.include, exclude = feed_config.exclude,
-                    as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs(),
-                    print_version_keywords = feed_config.get_print_version_keywords(),
-                    reformatter_url = feed_config.reformatter_url)
-                new_item_count = feed.update()
-                gobject.idle_add(self.feed_updated, feed_config, new_item_count)
+                urls = feed_config.get_urls()
+                title = None
+                if len(urls) == 1:
+                    title = feed_config.name
+                for url in urls:
+                    feed = feedcircuit.Feed(url = url, path = self.__model.cache_path, cache_path = ".cache",
+                        title = title, filename = feed_config.id + ".html",
+                        callback = self.callback, try_use_print_version = not feed_config.ignore_print_version,
+                        delete_before_date = date.today() - timedelta(feed_config.days_to_keep),
+                        inline_items = feed_config.mode == FeedConfig.mode_inline,
+                        cache_items = feed_config.mode == FeedConfig.mode_cache,
+                        include = feed_config.include, exclude = feed_config.exclude,
+                        as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs(),
+                        print_version_keywords = feed_config.get_print_version_keywords(),
+                        reformatter_url = feed_config.reformatter_url, next_page_keyword = feed_config.next_page_link)
+                    new_item_count = feed.update()
+                    gobject.idle_add(self.feed_updated, feed_config, new_item_count)
         finally:
             gobject.idle_add(self.update_complete)
             self.__update_lock.release()
@@ -906,7 +968,7 @@ class Program(hildon.Program):
             except Exception, ex:
                 pass
             feed_config =FeedConfig(self.__model.cache_path, url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0,
-                False, "", "", "")
+                False, "", "", "", "")
             edit_window = EditWindow(self.__window, feed_config)
             if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
@@ -980,7 +1042,7 @@ class Program(hildon.Program):
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.6")
+        dialog.set_version("0.6.7")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
